from abstra_internals.repositories.linter.rules.duplicate_package_in_requirements import (
    DuplicatePackagesInRequirements,
)
from abstra_internals.services.requirements import RequirementsRepository
from tests.fixtures import BaseTest


class TestDuplicatePackagesInRequirements(BaseTest):
    def test_default_valid(self):
        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 0)

    def test_valid_with_no_requirements_file(self):
        requirements_txt = self.root / "requirements.txt"
        requirements_txt.unlink()
        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 0)

    def test_valid_with_no_duplicates(self):
        requirements_txt = self.root / "requirements.txt"
        requirements_txt.write_text("abstra==1.0.0\nflask==1.0.0")
        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 0)

    def test_invalid_with_equal_duplicates(self):
        requirements_txt = self.root / "requirements.txt"
        requirements_txt.write_text("abstra==1.0.0\nabstra==1.0.0")
        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 1)
        self.assertEqual(len(issues[0].fixes), 1)

        issues[0].fixes[0].fix()

        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 0)

    def test_invalid_with_distinct_versions_choose_first(self):
        requirements_txt = self.root / "requirements.txt"
        requirements_txt.write_text("abstra==1.0.0\nabstra==2.0.0")
        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 1)
        self.assertEqual(len(issues[0].fixes), 2)

        issues[0].fixes[1].fix()

        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 0)

        requirements = RequirementsRepository.load()
        self.assertEqual(len(requirements.libraries), 1)
        self.assertEqual(requirements.libraries[0].name, "abstra")
        self.assertEqual(requirements.libraries[0].version, "1.0.0")

    def test_invalid_with_distinct_versions_choose_second(self):
        requirements_txt = self.root / "requirements.txt"
        requirements_txt.write_text("abstra==1.0.0\nabstra==2.0.0")
        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 1)
        self.assertEqual(len(issues[0].fixes), 2)

        issues[0].fixes[0].fix()

        issues = DuplicatePackagesInRequirements().find_issues()
        self.assertEqual(len(issues), 0)

        requirements = RequirementsRepository.load()
        self.assertEqual(len(requirements.libraries), 1)
        self.assertEqual(requirements.libraries[0].name, "abstra")
        self.assertEqual(requirements.libraries[0].version, "2.0.0")



from pathlib import Path

from dotenv import load_dotenv

from abstra_internals.controllers.main import MainController
from abstra_internals.logger import AbstraLogger
from abstra_internals.modules import reload_module


class FileHandler:
    def __init__(self, controller: MainController):
        self.controller = controller

    def env_var_handler(self, filepath: Path):
        if filepath.name == ".env":
            AbstraLogger.info("Reloading .env and all modules")
            load_dotenv(filepath, override=True)
            for (
                dep
            ) in self.controller.repositories.project.load().get_local_dependencies():
                reload_module(dep)
            return

    def python_files_handler(self, filepath: Path):
        if filepath.suffix == ".py":
            resolved_deps = [
                dep.resolve()
                for dep in self.controller.repositories.project.load().get_local_dependencies()
            ]

            if filepath.resolve() in resolved_deps:
                AbstraLogger.info(f"Reloading modified module: {filepath}")
                reload_module(filepath)
                return

    def all_files_handler(self, _: Path):
        self.controller.linter_repository.update_checks()



import threading
from abc import ABC, abstractmethod
from typing import List

from abstra_internals.repositories.linter.models import LinterCheck
from abstra_internals.repositories.linter.rules import rules


class LinterRepository(ABC):
    @abstractmethod
    def get_checks(self) -> List[LinterCheck]:
        pass

    @abstractmethod
    def update_checks(self):
        pass

    @abstractmethod
    def fix_linter(self, rule_name: str, fix_name: str) -> bool:
        pass

    @abstractmethod
    def fix_all_linters(self):
        pass

    @abstractmethod
    def get_blocking_checks(self) -> List[LinterCheck]:
        pass


def check_rule(rule, checks_list):
    check = rule.check()
    checks_list.append(check)


class LocalLinterRepository(LinterRepository):
    def __init__(self):
        self.checks: List[LinterCheck] = []

    def get_checks(self) -> List[LinterCheck]:
        return self.checks

    def update_checks(self):
        new_checks = []
        threads = []

        for rule in rules:
            thread = threading.Thread(target=check_rule, args=(rule, new_checks))
            thread.start()
            threads.append(thread)

        for thread in threads:
            thread.join()

        self.checks = new_checks

    def fix_linter(self, rule_name: str, fix_name: str):
        for check in self.checks:
            if check.name == rule_name:
                for issue in check.issues:
                    for fix in issue.fixes:
                        if fix.name == fix_name:
                            fix.fix()
                            return True
        return False

    def fix_all_linters(self):
        for check in self.checks:
            if check.type != "info":
                for issue in check.issues:
                    for fix in issue.fixes:
                        fix.fix()

    def get_blocking_checks(self) -> List[LinterCheck]:
        return [
            check
            for check in self.checks
            if check.type in ["error", "security", "bug"] and check.issues
        ]


class ProductionLinterRepository(LinterRepository):
    """
    This is a dummy repository. Linters are not available in production.
    """

    def get_checks(self) -> List[LinterCheck]:
        return []

    def update_checks(self):
        pass

    def fix_linter(self, rule_name: str, fix_name: str) -> bool:
        return False

    def fix_all_linters(self):
        pass

    def get_blocking_checks(self) -> List[LinterCheck]:
        return []



import json
from abc import ABC, abstractmethod
from typing import Any, List

from abstra_internals.cloud_api.http_client import HTTPClient
from abstra_internals.credentials import resolve_headers


class AIRepository(ABC):
    def __init__(self, client: "HTTPClient") -> None:
        self.client = client

    @abstractmethod
    def prompt(self, messages: List[Any], tools: List[Any], temperature: float):
        raise NotImplementedError()

    @abstractmethod
    def parse_document(self, model: str, file_content: bytes, mime_type: str):
        raise NotImplementedError()


class ProductionAIRepository(AIRepository):
    def prompt(self, messages: List[Any], tools: List[Any], temperature: float):
        body = {
            "messages": messages,
            "tools": tools,
            "temperature": temperature,
        }
        response = self.client.post(
            endpoint="/prompt",
            json=body,
        )
        try:
            response = response.json()
            return response
        except json.JSONDecodeError:
            raise Exception(f"Error parsing JSON: {response.text}")

    def parse_document(self, model: str, file_content: bytes, mime_type: str):
        response = self.client.post(
            endpoint=f"/parse-document/{model}",
            headers={"Content-Type": mime_type},
            data=file_content,
        )
        response.raise_for_status()

        return response.json()


class LocalAIRepository(AIRepository):
    def prompt(self, messages: List[Any], tools: List[Any], temperature: float):
        body = {
            "messages": messages,
            "tools": tools,
            "temperature": temperature,
        }

        response = self.client.post("/ai/prompt", json=body)

        response.raise_for_status()

        response = response.json()
        return response

    def parse_document(self, model: str, file_content: bytes, mime_type: str):
        headers = resolve_headers()
        if headers is None:
            raise Exception("You must be logged in to use AI")
        response = self.client.post(
            f"/ai/parse-document/{model}",
            headers={**headers, "Content-Type": mime_type},
            data=file_content,
        )
        response.raise_for_status()

        return response.json()



import threading
import time
from pathlib import Path
from typing import Callable, List, Optional

from watchdog.events import FileModifiedEvent, FileSystemEvent, FileSystemEventHandler
from watchdog.observers import Observer

from abstra_internals.settings import Settings

IGNORED_FILES = [
    ".abstra/resources.dat",
    ".abstra/",
    "abstra.json",
    ".venv",
    "__pycache__",
]

Handler = Callable[[Path], None]


class FileChangeWatcher(FileSystemEventHandler):
    def __init__(self):
        super().__init__()
        self._debounce_timer: Optional[threading.Timer] = None
        self.handlers: List[Handler] = []

    def add_handler(self, handler: Handler):
        self.handlers.append(handler)

    def on_modified(self, event: FileSystemEvent):
        filepath = Path(event.src_path)

        if self.should_ignore_event(event) or self.should_ignore_path(filepath):
            return

        threads = []
        for handler in self.handlers:
            thread = threading.Thread(target=handler, args=(filepath,))
            thread.start()
            threads.append(thread)

        for thread in threads:
            thread.join()

    def should_ignore_path(self, path: Path) -> bool:
        return any(ignored_file in str(path) for ignored_file in IGNORED_FILES)

    def should_ignore_event(self, event: FileSystemEvent) -> bool:
        return not isinstance(event, (FileModifiedEvent))

    def run(self):
        observer = Observer()
        observer.schedule(self, path=str(Settings.root_path), recursive=True)
        observer.start()

        try:
            while True:
                time.sleep(1)

        except KeyboardInterrupt:
            observer.stop()
        observer.join()
